@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
-from quodlibet.browsers.playlists.util import parse_m3u, parse_pls, PLAYLISTS
-from quodlibet.util.collection import Playlist
-from tests import TestCase, DATA_DIR, mkstemp, mkdtemp
-from helper import dummy_path
+from quodlibet.browsers.playlists.prefs import DEFAULT_PATTERN_TEXT
+from quodlibet.browsers.playlists.util import *
+from quodlibet.util.collection import FileBackedPlaylist
+from tests import TestCase, DATA_DIR, mkstemp, mkdtemp, _TEMP_DIR
+from .helper import dummy_path
 
 import os
 import shutil
@@ -30,7 +31,7 @@ class TParsePlaylistMixin(object):
     def test_parse_empty(self):
         h, name = mkstemp()
         os.close(h)
-        file(name, "w").close()
+        open(name, "w").close()
         pl = self.Parse(name)
         os.unlink(name)
         self.failUnlessEqual(0, len(pl))
@@ -56,9 +57,8 @@ class TParsePlaylistMixin(object):
         from quodlibet.util.uri import URI
         target = URI.frompath(target)
         target = self.prefix + target
-        f = file(name, "w")
-        f.write(target)
-        f.close()
+        with open(name, "w") as f:
+            f.write(target)
         list = self.Parse(name)
         os.unlink(name)
         self.failUnlessEqual(len(list), 1)
@@ -109,7 +109,7 @@ class TPlaylistIntegration(TestCase):
             af.sanitize()
         self.lib.add(self.SONGS)
         self._dir = mkdtemp()
-        self.pl = Playlist.new(self._dir, "Foobar", self.lib)
+        self.pl = FileBackedPlaylist.new(self._dir, "Foobar", self.lib)
         self.pl.extend(self.SONGS)
 
     def tearDown(self):
@@ -145,14 +145,14 @@ class TPlaylistIntegration(TestCase):
         self.failUnlessEqual(len(self.pl), len(self.SONGS) - 1)
 
     def test_remove_no_lib(self):
-        pl = Playlist.new(self._dir, "Foobar")
+        pl = FileBackedPlaylist.new(self._dir, "Foobar")
         pl.extend(self.SONGS)
         self.assertTrue(len(pl))
         pl.remove_songs(self.SONGS, False)
         self.assertFalse(len(pl))
 
 
-class TPlaylists(TSearchBar):
+class TPlaylistsBrowser(TSearchBar):
     Bar = PlaylistsBrowser
 
     ANOTHER_SONG = AudioFile({
@@ -161,6 +161,9 @@ class TPlaylists(TSearchBar):
         "~filename": dummy_path(u"/dev/urandom")})
 
     def setUp(self):
+        # Testing locally is VERY dangerous without this...
+        self.assertTrue(_TEMP_DIR in PLAYLISTS or os.name == "nt",
+                        msg="Failing, don't want to delete %s" % PLAYLISTS)
         try:
             shutil.rmtree(PLAYLISTS)
         except OSError:
@@ -175,11 +178,11 @@ class TPlaylists(TSearchBar):
             af.sanitize()
         self.lib.add(all_songs)
 
-        pl = Playlist.new(PLAYLISTS, "Big", self.lib)
+        self.big = pl = FileBackedPlaylist.new(PLAYLISTS, "Big", self.lib)
         pl.extend(SONGS)
         pl.write()
 
-        pl = Playlist.new(PLAYLISTS, "Small", self.lib)
+        self.small = pl = FileBackedPlaylist.new(PLAYLISTS, "Small", self.lib)
         pl.extend([self.ANOTHER_SONG])
         pl.write()
 
@@ -234,3 +237,14 @@ class TPlaylists(TSearchBar):
         self.bar.filter_text("piman")
         self.assertFalse(self.bar.active_filter(self.ANOTHER_SONG),
                          msg="Shouldn't have matched 'piman' on second list")
+
+    def test_rename(self):
+        self.assertEquals(self.bar.playlists()[1], self.small)
+        self.bar._rename(0, "zBig")
+        self.assertEquals(self.bar.playlists()[0], self.small)
+        self.assertEquals(self.bar.playlists()[1].name, "zBig")
+
+    def test_default_display_pattern(self):
+        pattern_text = self.bar.display_pattern_text
+        self.failUnlessEqual(pattern_text, DEFAULT_PATTERN_TEXT)
+        self.failUnless("<~name>" in pattern_text)
