@@ -10,7 +10,7 @@ import os
 import pickle
 
 from tests import TestCase, DATA_DIR
-from helper import capture_output, temp_filename
+from .helper import capture_output, temp_filename
 
 from quodlibet import formats
 from quodlibet.formats import AudioFile
