@@ -27,6 +27,10 @@ class _TPattern(AbstractTestCase):
         s7 = {'artist': u'un élève français', '~filename': '/path/to/g.mp3',
               'albumartist': u'Lee "Scratch" Perry',
               'album': "The 'only' way!", 'comment': 'Trouble|Strife'}
+        s8 = {'tracknumber': '7/8', 'artist': 'Artist1\n\nArtist3',
+              'artistsort': 'SortA1\nSortA2',
+              'album': 'Album5', 'albumsort': 'SortAlbum5',
+              '~filename': '/path/to/g.mp3', 'xmltest': "<&>"}
 
         if os.name == "nt":
             s1["filename"] = u"C:\\path\\to\\a.mp3"
@@ -36,6 +40,7 @@ class _TPattern(AbstractTestCase):
             s5["filename"] = u"C:\\path\\to\\a.mp3"
             s6["filename"] = u"C:\\path\\to\\f.mp3"
             s7["filename"] = u"C:\\path\\to\\g.mp3"
+            s8["filename"] = u"C:\\path\\to\\h.mp3"
 
         self.a = AudioFile(s1)
         self.b = AudioFile(s2)
@@ -44,12 +49,32 @@ class _TPattern(AbstractTestCase):
         self.e = AudioFile(s5)
         self.f = AudioFile(s6)
         self.g = AudioFile(s7)
+        self.h = AudioFile(s8)
 
 
 class TPattern(_TPattern):
     from quodlibet.formats import AudioFile
     AudioFile
 
+    def test_numeric(self):
+        pat = Pattern("<~#rating>")
+        self.assertEqual(pat.format(self.a), "0.50")
+
+    def test_space(self):
+        pat = Pattern("a ")
+        self.assertEqual(pat.format(self.a), "a ")
+        pat = Pattern(" a")
+        self.assertEqual(pat.format(self.a), " a")
+        pat = Pattern("a\n\n")
+        self.assertEqual(pat.format(self.a), "a\n\n")
+
+    def test_escape(self):
+        pat = Pattern("a \\<foo\\|bla\\>")
+        self.assertEqual(pat.format(self.a), "a <foo|bla>")
+
+        pat = Pattern(r"a\\<foo>")
+        self.assertEqual(pat.format(self.a), "a\\")
+
     def test_query_like_tag(self):
         pat = Pattern("<t=v>")
         self.assertEqual(pat.format(AudioFile({"t=v": "foo"})), "foo")
@@ -124,6 +149,12 @@ class TPattern(_TPattern):
         pat = Pattern("</The .+ way/|matched|not matched>")
         s.assertEquals(pat.format(s.g), 'not matched')
 
+    def test_tag_internal(self):
+        pat = Pattern("<~filename='/path/to/a.mp3'|matched|not matched>")
+        self.assertEquals(pat.format(self.a), 'matched')
+        pat = Pattern("<~filename=/\\/path\\/to\\/a.mp3/|matched|not matched>")
+        self.assertEquals(pat.format(self.a), 'matched')
+
     def test_tag_query_disallowed_free_text(s):
         pat = Pattern("<The only way|matched|not matched>")
         s.assertEquals(pat.format(s.g), 'not matched')
@@ -197,12 +228,15 @@ class _TFileFromPattern(_TPattern):
 
     def test_escape_slash(s):
         fpat = s._create('<~filename>')
-        pat = Pattern('<~filename>')
-        wpat = s._create(r'\\<artist>\\ "<title>')
         s.assertTrue(fpat.format(s.a).startswith("_path_to_a.mp3"))
+
+        pat = Pattern('<~filename>')
         s.assertTrue(pat.format(s.a).startswith("/path/to/a"))
+
         if os.name != "nt":
-            s.assertTrue(wpat.format(s.a).startswith("\\Artist\\ \"Title5"))
+            wpat = s._create(r'\\<artist>\\ "<title>')
+            s.assertTrue(
+                wpat.format(s.a).startswith(r'\Artist\ "Title5'))
         else:
             # FIXME..
             pass
@@ -378,8 +412,8 @@ class TXMLFromMarkupPattern(_TPattern):
         s.assertEquals(pat.format(s.a), '[b]')
         s._test_markup(pat.format(s.a))
 
-        pat = XMLFromMarkupPattern(r'\\[b]\\[/b]')
-        s.assertEquals(pat.format(s.a), r'\<b>\</b>')
+        pat = XMLFromMarkupPattern(r'\\\\[b]\\\\[/b]')
+        s.assertEquals(pat.format(s.a), r'\\<b>\\</b>')
         s._test_markup(pat.format(s.a))
 
 
@@ -396,40 +430,133 @@ class TRealTags(TestCase):
 
 
 class TPatternFormatList(_TPattern):
+
+    def test_numeric(self):
+        pat = Pattern("<~#rating>")
+        self.assertEqual(pat.format_list(self.a), {("0.50", "0.50")})
+
+    def test_empty(self):
+        pat = Pattern("<nopenope>")
+        self.assertEqual(pat.format_list(self.a), {("", "")})
+
     def test_same(s):
         pat = Pattern('<~basename> <title>')
-        s.failUnlessEqual(pat.format_list(s.a), {pat.format(s.a)})
+        s.failUnlessEqual(pat.format_list(s.a),
+                          {(pat.format(s.a), pat.format(s.a))})
         pat = Pattern('/a<genre|/<genre>>/<title>')
-        s.failUnlessEqual(pat.format_list(s.a), {pat.format(s.a)})
+        s.failUnlessEqual(pat.format_list(s.a),
+                          {(pat.format(s.a), pat.format(s.a))})
 
     def test_same2(s):
         fpat = FileFromPattern('<~filename>')
         pat = Pattern('<~filename>')
-        s.assertEquals(fpat.format_list(s.a), {fpat.format(s.a)})
-        s.assertEquals(pat.format_list(s.a), {pat.format(s.a)})
+        s.assertEquals(fpat.format_list(s.a),
+                       {(fpat.format(s.a), fpat.format(s.a))})
+        s.assertEquals(pat.format_list(s.a),
+                       {(pat.format(s.a), pat.format(s.a))})
 
     def test_tied(s):
         pat = Pattern('<genre>')
-        s.failUnlessEqual(pat.format_list(s.c), {'/', '/'})
+        s.failUnlessEqual(pat.format_list(s.c), {('/', '/')})
         pat = Pattern('<performer>')
-        s.failUnlessEqual(pat.format_list(s.d), {'a', 'b'})
+        s.failUnlessEqual(pat.format_list(s.d), {('a', 'a'), ('b', 'b')})
         pat = Pattern('<performer><performer>')
         s.failUnlessEqual(set(pat.format_list(s.d)),
-                          {'aa', 'ab', 'ba', 'bb'})
+                          {('aa', 'aa'), ('ab', 'ab'),
+                           ('ba', 'ba'), ('bb', 'bb')})
         pat = Pattern('<~performer~artist>')
         s.failUnlessEqual(pat.format_list(s.d),
-                          {'a - foo', 'b - foo', 'a - bar', 'b - bar'})
+                          {('a', 'a'), ('b', 'b'),
+                           ('bar', 'bar'), ('foo', 'foo')})
         pat = Pattern('<performer~artist>')
         s.failUnlessEqual(pat.format_list(s.d),
-                          {'a - foo', 'b - foo', 'a - bar', 'b - bar'})
+                          {('a', 'a'), ('b', 'b'),
+                           ('bar', 'bar'), ('foo', 'foo')})
         pat = Pattern('<artist|<artist>.|<performer>>')
-        s.failUnlessEqual(pat.format_list(s.d), {'foo.', 'bar.'})
+        s.failUnlessEqual(pat.format_list(s.d),
+                          {('foo.', 'foo.'), ('bar.', 'bar.')})
         pat = Pattern('<artist|<artist|<artist>.|<performer>>>')
-        s.failUnlessEqual(pat.format_list(s.d), {'foo.', 'bar.'})
+        s.failUnlessEqual(pat.format_list(s.d),
+                          {('foo.', 'foo.'), ('bar.', 'bar.')})
+
+    def test_sort(s):
+        pat = Pattern('<album>')
+        s.failUnlessEqual(pat.format_list(s.f),
+                          {(u'Best Of', u'Best Of')})
+        pat = Pattern('<album>')
+        s.failUnlessEqual(pat.format_list(s.h), {(u'Album5', u'SortAlbum5')})
+        pat = Pattern('<artist>')
+        s.failUnlessEqual(pat.format_list(s.h), {(u'Artist1', u'SortA1'),
+                                                 (u'', u'SortA2'),
+                                                 (u'Artist3', u'Artist3')})
+        pat = Pattern('<artist> x')
+        s.failUnlessEqual(pat.format_list(s.h), {(u'Artist1 x', u'SortA1 x'),
+                                                 (u' x', u'SortA2 x'),
+                                                 (u'Artist3 x', u'Artist3 x')})
+
+    def test_sort_tied(s):
+        pat = Pattern('<~artist~album>')
+        s.failUnlessEqual(pat.format_list(s.h), {(u'Artist1', u'SortA1'),
+                                                 (u'', u'SortA2'),
+                                                 (u'Artist3', u'Artist3'),
+                                                 (u'Album5', u'SortAlbum5')})
+        pat = Pattern('<~album~artist>')
+        s.failUnlessEqual(pat.format_list(s.h), {(u'Artist1', u'SortA1'),
+                                                 (u'', u'SortA2'),
+                                                 (u'Artist3', u'Artist3'),
+                                                 (u'Album5', u'SortAlbum5')})
+        pat = Pattern('<~artist~artist>')
+        s.failUnlessEqual(pat.format_list(s.h), {(u'Artist1', u'SortA1'),
+                                                 (u'', u'SortA2'),
+                                                 (u'Artist3', u'Artist3')})
+
+    def test_sort_combine(s):
+        pat = Pattern('<album> <artist>')
+        s.failUnlessEqual(pat.format_list(s.h),
+                          {(u'Album5 Artist1', u'SortAlbum5 SortA1'),
+                           (u'Album5 ', u'SortAlbum5 SortA2'),
+                           (u'Album5 Artist3', u'SortAlbum5 Artist3')})
+        pat = Pattern('x <artist> <album>')
+        s.failUnlessEqual(pat.format_list(s.h),
+                          {(u'x Artist1 Album5', u'x SortA1 SortAlbum5'),
+                           (u'x  Album5', u'x SortA2 SortAlbum5'),
+                           (u'x Artist3 Album5', u'x Artist3 SortAlbum5')})
+        pat = Pattern(' <artist> <album> xx')
+        s.failUnlessEqual(pat.format_list(s.h),
+                          {(u' Artist1 Album5 xx', u' SortA1 SortAlbum5 xx'),
+                           (u'  Album5 xx', u' SortA2 SortAlbum5 xx'),
+                           (u' Artist3 Album5 xx', u' Artist3 SortAlbum5 xx')})
+        pat = Pattern('<album> <tracknumber> <artist>')
+        s.failUnlessEqual(pat.format_list(s.h),
+                          {(u'Album5 7/8 Artist1', u'SortAlbum5 7/8 SortA1'),
+                           (u'Album5 7/8 ', u'SortAlbum5 7/8 SortA2'),
+                           (u'Album5 7/8 Artist3', u'SortAlbum5 7/8 Artist3')})
+        pat = Pattern('<tracknumber> <album> <artist>')
+        s.failUnlessEqual(pat.format_list(s.h),
+                          {(u'7/8 Album5 Artist1', u'7/8 SortAlbum5 SortA1'),
+                           (u'7/8 Album5 ', u'7/8 SortAlbum5 SortA2'),
+                           (u'7/8 Album5 Artist3', u'7/8 SortAlbum5 Artist3')})
+
+    def test_sort_multiply(s):
+        pat = Pattern('<artist> <artist>')
+        s.failUnlessEqual(pat.format_list(s.h),
+                          {(u'Artist1 Artist1', u'SortA1 SortA1'),
+                           (u' Artist1', u'SortA2 SortA1'),
+                           (u'Artist3 Artist1', u'Artist3 SortA1'),
+                           (u'Artist1 ', u'SortA1 SortA2'),
+                           (u' ', u'SortA2 SortA2'),
+                           (u'Artist3 ', u'Artist3 SortA2'),
+                           (u'Artist1 Artist3', u'SortA1 Artist3'),
+                           (u' Artist3', u'SortA2 Artist3'),
+                           (u'Artist3 Artist3', u'Artist3 Artist3')})
 
     def test_missing_value(self):
         pat = Pattern('<genre> - <artist>')
-        self.assertEqual(pat.format_list(self.a), {" - Artist"})
-
+        self.assertEqual(pat.format_list(self.a),
+                         {(" - Artist", " - Artist")})
         pat = Pattern('')
-        self.assertEqual(pat.format_list(self.a), {""})
+        self.assertEqual(pat.format_list(self.a), {("", "")})
+
+    def test_string(s):
+        pat = Pattern('display')
+        s.assertEqual(pat.format_list(s.a), {("display", "display")})
